# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2024 CERN.
#
# Invenio-Vocabularies is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio module for managing vocabularies."""

from .ext import InvenioVocabularies

__version__ = "6.2.0"

__all__ = ("__version__", "InvenioVocabularies")
